/****************************************************************************
* zbedic.cpp
*
* Copyright (C) 2002 Latchesar Ionkov <lionkov@yahoo.com>
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
*
* This program is based on the kbedic dictionary by 
* Radostin Radnev <radnev@yahoo.com>.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>

#include <qwidget.h>
#include <qmainwindow.h>
#include <qapplication.h>
#include <qfont.h>
#include <qclipboard.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qtextview.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qstring.h>

#include <qmenubar.h>
#include <qlayout.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qaction.h>
#include <qsizepolicy.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qvgroupbox.h>
#include <qtextbrowser.h>
#include <qlist.h>
#include <qdir.h>
#include <qaccel.h>

#include <qpe/qcopenvelope_qws.h>
#include <qpe/config.h>
#include <qpe/resource.h>
#include <qpe/fontdatabase.h>
#include <qpe/global.h>
#include <qpe/applnk.h>

#include <sys/time.h>

#include "zbedic.h"
#include "dictionary.h"
#include "utf8.h"

#include <assert.h>

using namespace std;

//void zprint(void);

ZBEDic::ZBEDic(QWidget *parent, const char *name, WFlags f) :
	QMainWindow(parent, name, f), dict( NULL ), onWordLookup( false ),
        fullyInitialized(false) {

  Config conf("zbedic");
  conf.setGroup("general");

  // Read data from ini file and set it
  layoutMode = conf.readNumEntry("layout-mode", 0);
  listLen = conf.readNumEntry("list-length", 25);
  wordListSize = conf.readNumEntry("word-list-size", 1);
  historyLen = conf.readNumEntry("history-length", 30);
  showPron = conf.readBoolEntry("show-pron", true);
  previousDict = conf.readEntry("previous-dictionary", QString::null);


  // Set font, read from file settings and set it
  fontName = conf.readEntry("font-name", "unifont");
  fontSize = conf.readNumEntry("font-size", 11);
  font = QFont(fontName, fontSize);
  updateFont(font);

// 	dictPaths = conf.readListEntry("dict-path", ':');
// 	if (dictPaths.count() == 0) {
// 		dictPaths.append("/opt/QtPalmtop/share/zbedic");
// 		dictPaths.append("/mnt/cf/opt/QtPalmtop/share/zbedic");
// 		dictPaths.append("/mnt/card/opt/QtPalmtop/share/zbedic");
// 	}

  // Load the dictionaries
  QStringList activeDicts = conf.readListEntry("active-dictionaries", ':');
  loadDictionaries( activeDicts );

  descrFormatter = new DescrFormatter();
  descrFormatter->setShowPron(showPron);
  setCaption(tr("ZBEDic Dictionary"));

  // create the toolbar
  QToolBar* toolBar = new QToolBar(this);
  toolBar->setHorizontalStretchable(TRUE);
  setToolBarsMovable(FALSE);

  wordBox = new WordBox(toolBar);
  CHECK_PTR(wordBox);
  wordBox->setEditable(true);
  wordBox->setInsertionPolicy(QComboBox::NoInsertion);
  wordBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred, false));

  word = wordBox->lineEdit();
//	wordBox->setFixedWidth(120);
  toolBar->setStretchableWidget(wordBox);

  // restore the values in the history
  QListBox* lb = wordBox->listBox();
  QStringList history = conf.readListEntry("history", ',');
  for(unsigned int i = 0; i < history.count(); i++) {
    int atSignPos = history[i].findRev( '@' );
    lb->insertItem(new HistoryItem( history[i].left( atSignPos ),
                     history[i].mid( atSignPos+1 ) ));          
  }
  int historyCurrent = conf.readNumEntry( "history-current", 0 );
  wordBox->setCurrentItem( historyCurrent );

  dictButton = new QToolButton(Resource::loadPixmap("zbedic/dictionary"), tr("Select Dictionary"),
    QString::null, this, SLOT(showDictionaries()), toolBar);	

  dictPopup = new QPopupMenu();
  fillInDictPopup();


  toolBar->addSeparator();

  toolButton[TB_SwitchDictionaries] = new QToolButton(Resource::loadPixmap("zbedic/switch_dictionary"), tr("Switch Dictionaries"),
    QString::null, this, SLOT(switchToPreviousDict()), toolBar);	

  toolButton[TB_Back] = new QToolButton(Resource::loadPixmap("zbedic/back"), tr("Back"),
    QString::null, this, SLOT(navigationPrevious()), toolBar);	

  toolButton[TB_Forward] = new QToolButton(Resource::loadPixmap("zbedic/forward"), tr("Forward"),
    QString::null, this, SLOT(navigationNext()), toolBar);	
  
  toolButton[TB_LookupSelection] = new QToolButton(Resource::loadPixmap("zbedic/lookup_selection"),
    tr("LookupSelection"), QString::null, this, SLOT(lookupSelection()), toolBar);

  toolButton[TB_Copy] = new QToolButton(Resource::loadPixmap("copy"), tr("Copy"),
    QString::null, this, SLOT(copy()), toolBar);

  toolButton[TB_Paste] = new QToolButton(Resource::loadPixmap("paste"), tr("Paste"),
    QString::null, this, SLOT(paste()), toolBar);
        
//  settingsButton = new QToolButton(Resource::loadPixmap("SettingsIcon"), tr("Settings"),
//    QString::null, this, SLOT(settings()), toolBar);

  cancelButton = new QToolButton(Resource::loadPixmap("editdelete"), tr("Hide"),
    QString::null, this, SLOT(cancel()), toolBar);

  cancelButton->hide();

  if( conf.hasKey("tool-bar") ) {
    QStringList toolBarConfig = conf.readListEntry("tool-bar", ';');
    for( int i = 0; i < TB_count; i++ ) {
      if( toolBarConfig.findIndex( toolButton[i]->textLabel() ) == -1 )
        toolButton[i]->hide();
    }
  }        
        
  QWidget* w = new QWidget(this);

  setCentralWidget(w);
  mainLayout = new QBoxLayout(w, QBoxLayout::TopToBottom);
  mainLayout->setSpacing(4);
  mainLayout->setMargin(2);

  // Create and set list (of words)
  wordList = new WordList(w);
  CHECK_PTR(wordList);
  wordList->setHScrollBarMode(QScrollView::AlwaysOff );
  wordList->setFocusPolicy(QWidget::NoFocus);
//  wordList->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
  mainLayout->addWidget(wordList, 1);

  descr = new DictBrowser(w);
  CHECK_PTR(descr);
  descr->setTextFormat(QTextView::RichText);
  descr->setFocusPolicy( QWidget::ClickFocus );
  
//  descr->setResizePolicy(QScrollView::AutoOneFit);
//  descr->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
  mainLayout->addWidget(descr, 2);

  updateLayout( wordListSize, layoutMode );

  // restore current dictionary
  QString dname = conf.readEntry("dictionary", QString::null);
  Dictionary* d = findDictionary(dname);
  if (d == 0) {
    if (dictionaries.count() > 0) {
      d = dictionaries.at(0);
    }
  }

  setDictionary(d);

  // TODO: Fixme: Nasty trick to make the size of word list OK
  word->clear();
  wordList->clear();        
		
  // get application clipboard
  systemClipboard = QApplication::clipboard();

  word->setFocus();

  if (dict!= 0) {
    // restore the state
    QString w = conf.readEntry("word");
    updateWordLookup( w, true, true );
//    word->setText( w );
//    wordChanged( w );
  }

  addToHistoryTimer = new QTimer( this );
  connect(addToHistoryTimer, SIGNAL(timeout()), 
    this, SLOT(addToHistoryTriggered()));

        
  //Accelerators
  QAccel *accel = new QAccel( this );  
  accel->connectItem( accel->insertItem( '#' + CTRL ),
    this, SLOT( showDictionaries( void ) ) );
  accel->connectItem( accel->insertItem( '_' + CTRL ),
    wordBox, SLOT( activatePopup( void ) ) );
  accel->connectItem( accel->insertItem( '@' + CTRL ),
    this, SLOT( switchToPreviousDict() ) );
  accel->connectItem( accel->insertItem( '-' + CTRL ),
    descr, SLOT( setFocus() ) );
  
  connect(word, SIGNAL(textChanged(const QString&)), 
    this, SLOT(wordChanged(const QString&)));

//   connect(wordBox, SIGNAL(activated(int)), 
//     this, SLOT(historyItemActivated(int)));
  
  connect(dictPopup, SIGNAL(activated(int)), 
    this, SLOT(selectDictionary(int)));

  connect(wordList, SIGNAL(highlighted(const QString&)), 
    this, SLOT(listPick(const QString&)));

  connect(descr, SIGNAL(anchor(const QString&)), 
    this, SLOT(seeAlso(const QString&)));

  adjustSize();

  QObject::connect(qApp, SIGNAL(appMessage(const QCString&, const QByteArray&)), 
    this, SLOT(msgHandler(const QCString&, const QByteArray&)));
  
  QCopChannel* c1 = new QCopChannel("QPE/Card", this);
  QObject::connect(c1, SIGNAL(received(const QCString&, const QByteArray&)),
    this, SLOT(c1Handler(const QCString&, const QByteArray&)));
  
  QCopChannel* c2 = new QCopChannel("QPE/System", this);
  QObject::connect(c2, SIGNAL(received(const QCString&, const QByteArray&)),
    this, SLOT(c2Handler(const QCString&, const QByteArray&)));
  
  fullyInitialized = true;
}


// void ZBEDic::polish()
// {
//   QMainWindow::polish();

//   Config conf("zbedic");
//   conf.setGroup("general");

//   QString w = conf.readEntry("word");
//   word->setText( w );
//   wordChanged( w );
  
// }


void ZBEDic::closeEvent( QCloseEvent * e )
{
  // Save config and exit
  Config conf("zbedic");
  conf.setGroup("general");

  conf.writeEntry("word", word->text());

  // save the history
  QStringList history;
  QListBox* list = wordBox->listBox();
  for(unsigned int i = 0; i < list->count(); i++) {
    HistoryItem *item = (HistoryItem*)list->item( i );
    const QString &itemDict = item->getDictionary();
    history.append(list->text(i) + "@" + itemDict );
  }
  conf.writeEntry("history", history, ',');
  conf.writeEntry("history-current", wordBox->currentItem() );
        
  QStringList activeDicts;
  for(unsigned int i = 0; i < dictionaries.count(); i++) {
    activeDicts.append( dictionaries.at(i)->getFileName().c_str() );
  }
  conf.writeEntry("active-dictionaries", activeDicts, ':');

  conf.writeEntry("layout-mode", layoutMode);
  conf.writeEntry("word-list-size", wordListSize);
  conf.writeEntry("history-length", (int) historyLen);
  conf.writeEntry("list-length", (int) listLen);
  conf.writeEntry("previous-dictionary", previousDict);
//	conf.writeEntry("dict-path", dictPaths, ':');
  if (dict != 0) {
    conf.writeEntry("dictionary", QString(dict->getName().c_str()));
  }

  conf.writeEntry("font-name", fontName);
  conf.writeEntry("font-size", fontSize);

  conf.writeEntry("show-pron", showPron);

  QStringList toolBarConfig;
  for( int i = 0; i < TB_count; i++ ) {
    if( toolButton[i]->isVisible() )
      toolBarConfig.append(toolButton[i]->textLabel());
  }
  conf.writeEntry("tool-bar", toolBarConfig, ';');

  QMainWindow::closeEvent( e );
}

ZBEDic::~ZBEDic() {
        
  if (descrFormatter) {
    delete descrFormatter;
  }

  if (dictPopup) {
    delete dictPopup;
  }

  for(unsigned int i = 0; i < dictionaries.count(); i++) {
    delete dictionaries.at(i);
  }
}

void ZBEDic::wordChanged(const QString& text)
{
  if( onWordLookup ) return;

//  saveScrollPosition();

//  printf( "Word changed %s\n", text.latin1() );

  if( isCurrentHistoryItem( text ) ) { // If selected from the history
    HistoryItem *item = getCurrentHistoryItem();
//    printf( "Is history item: %s\n", item->text().latin1() );

//    descr->viewport()->setUpdatesEnabled( FALSE );
    
    Dictionary *itemDictionary = findDictionary( item->getDictionary() );
    if( itemDictionary != NULL && itemDictionary != dict ) {
      setDictionary( itemDictionary );
    } else {
      updateWordLookup( text, TRUE, FALSE );
    }      

    // Those three calls seems to be the only way to make scrolling in
    // large entries work on SL5500
    qApp->processEvents();
    qApp->processEvents();
    qApp->processEvents();

//    printf( "Height after update: %d\n", descr->contentsHeight() );

    descr->setContentsPos( descr->contentsX(), item->getScrollPosition() );

//     printf( "Item: %s Going to pos: %d, achieved: %d\n", item->text().latin1(),
//       item->getScrollPosition(), descr->contentsY()  );
    
//    descr->viewport()->setUpdatesEnabled( TRUE );
    
    return;
  }
  
  if( updateWordLookup( text, TRUE, FALSE ) )
    addToHistoryDelayed( text );
}

void ZBEDic::listPick(const QString& text)
{
  if( onWordLookup ) return;

  saveScrollPosition();
  
  updateWordLookup( text, FALSE, TRUE );
  addToHistoryDelayed( text );

  word->selectAll();
  word->setFocus();
}

// =============== Lookup for a word ================

bool ZBEDic::updateWordLookup( const QString &newWord, bool updateWordList, bool updateWordBox )
{
  onWordLookup = true;
    
  bool subword = false;
  bool exactMatch = false;
  QString w;
  QString m;
  
  if( updateWordBox )
    word->setText(newWord);
  
  // Look up dictionary
  if (dict != 0) {
    exactMatch = dict->findEntry(string(newWord.utf8()), subword);
    w = QString::fromUtf8(dict->getWord().c_str());
    m = QString::fromUtf8(dict->getSense().c_str());
  }

  // Set description if exact match
  QString formattedDescription;
  if( exactMatch ) {
    formattedDescription = descrFormatter->format(m, w);
    descrWord = w;              // newWord and descrWord may differ due to canonization
    descrText = formattedDescription;
    descr->setText(descrText);
  } else {
    descrWord = QString::null;
    descrText = QString::null;
    descr->setText(descrText);
  }
  
  // Fill out the list of words if update needed
  if( updateWordList &&
    (wordList->count()==0 || wordList->text(0) != w) &&
    dict != 0 ) {
          
    if (w.isEmpty()) {
      dict->firstEntry();
    }
          
    wordList->clear();
//          lword = QString::fromUtf8(dict->getWord().c_str());
          
    unsigned int i = 0;
    do {  
      wordList->insertItem(QString::fromUtf8(dict->getWord().c_str()));
      i++;
    } while (dict->nextEntry() && (i < listLen));
    wordList->setTopItem(0);                        
  }

  // Select a word from the list if exact match
  if( updateWordList ) {
    if( exactMatch )
      wordList->setSelected( 0, true ); // exact match must be 1st item
    else { // Deselect
      int item = wordList->currentItem();
      if( item != -1 ) wordList->setSelected( item, false );
    }
  }        

  updateNavigationButtons();
  
  onWordLookup = false;
  return exactMatch;
}

void ZBEDic::copy()
{
  if (descr->hasSelectedText()) {
    descr->copy();
  } else if (word->hasMarkedText()) {
    word->copy();
  }
}

void ZBEDic::paste() {
  word->setFocus();
  word->clear();
//	dict->randomEntry();
//	word->setText(QString::fromUtf8(dict->getWord().c_str()));
  word->paste();
}

void ZBEDic::cancel() {
  hide();
  cancelButton->hide();
  settingsButton->show();
}

void ZBEDic::selectDictionary(int i) {
  Dictionary *d;
  if( dictionaries.count() == 0 )
    d = NULL;
  else
    d = dictionaries.at(i);

  setDictionary( d );        
}

Dictionary* ZBEDic::findDictionary(const QString& name) {
  unsigned int i;
  Dictionary* d = 0;

  for(i = 0; i < dictionaries.count(); i++) {
    if (name == dictionaries.at(i)->getName().c_str()) {
      d = dictionaries.at(i);
      break;
    }
  }
	
  return d;
}

void ZBEDic::setDictionary(Dictionary* d) {

  if( dict != NULL )
    previousDict = dict->getName().c_str();
      
  dict = d;
  for(uint i = 0; i < dictionaries.count(); i++) {
    dictPopup->setItemChecked(i, d == dictionaries.at(i));
  }

  if (dict != 0) {
    setCaption("zbedic: " + QString(dict->getName().c_str()));
    if( updateWordLookup( word->text(), TRUE, FALSE ) )
      addToHistoryDelayed( word->text() );
  } else {
    wordList->clear();
    descr->setText("<qml><p>No dictionaries found. The reason may be:</p>"
      "<ul>"
      "<li>Dictionaries are installed, but not activated. Go to the settings dialog, "
      "Dictionaries tab, search for new dictionaries and make active the dictionaries "
      "you want to use. You can also open dictionary files from Document Manager to "
      "activate them.</li>"
      "<li>No dictionaries were installed. You can download "
      "dictionaries from "
      "http://www.sourceforge.net/projects/bedic</li>");
  }	

}

void ZBEDic::showDictionaries() {
  QPoint p = dictButton->mapToGlobal(QPoint(0, dictButton->height()));
  dictPopup->exec(p);
}


void ZBEDic::seeAlso(const QString& text) {
  findWord(text);
}

void ZBEDic::findWord(const QString& text)
{
  addToHistoryTriggered();      // Allow for back navigation
  
  saveScrollPosition();

  if( updateWordLookup( text ) )
    addToHistory( text );
}


// ============= History ===========
bool ZBEDic::isCurrentHistoryItem( const QString &word, const Dictionary *dic )
{ 
  HistoryItem *item = getCurrentHistoryItem();
  if( item == NULL ) return false;

//   if( dic != NULL )
//     printf( "'%s' '%s'\n", dic->getName().c_str(), item->getDictionary().latin1() );
  
  if( dic != NULL && dic->getName() != item->getDictionary().latin1() )
    return false;

//  printf( "Current hist: '%s',  word: '%s'\n", item->text().latin1(), word.latin1() );

  return item->text() == word;
}

HistoryItem *ZBEDic::getCurrentHistoryItem()
{
  if( wordBox->currentItem() < 0 || wordBox->currentItem() >= (int)wordBox->count() ) return NULL;
  return (HistoryItem*)wordBox->listBox()->item( wordBox->currentItem() );
}

void ZBEDic::saveScrollPosition()
{
  // For backward navigation: if previous word is in history, save scroll position
  // of the description box
  if( isCurrentHistoryItem( word->text() ) ) {
    getCurrentHistoryItem()->setScrollPosition( descr->contentsY() );
//    printf( "Scroll pos saved: %d\n", descr->contentsY() );
  }
}

// void ZBEDic::historyItemActivated(int idx)
// {
//    printf( "hist item activ %d\n", idx );
// //   QListBox* list = wordBox->listBox();
// //   HistoryItem *item = (HistoryItem*)list->item( idx );
// //   gotoHistoryItem( item );   
// }


void ZBEDic::addToHistoryTriggered()
{
  addToHistoryTimer->stop();
  if( wordToAddToHistory.compare( wordBox->currentText() ) != 0 ) return;
  addToHistory( wordToAddToHistory );
  wordToAddToHistory = QString::null;  
}


void ZBEDic::addToHistory(const QString& s) {
  QListBox* list = wordBox->listBox();
  
//  printf( "Add to hist\n" );

//   for(unsigned int i = 0; i < list->count(); i++) {
//     HistoryItem* item = (HistoryItem*)list->item(i);
//     if (s == item->text()) {
//       return;
//     }
//   }

  HistoryItem *newItem = new HistoryItem( s, dict->getName().c_str() );

  int insertAt = wordBox->currentItem();
  if( insertAt == -1 ) insertAt = 0;

  if( getCurrentHistoryItem() != NULL ) {
    // Special case - don't add to history if current or next entry is
    // the same as the new entry
    if( getCurrentHistoryItem()->text() == s ) return;

    if( insertAt > 0 && wordBox->listBox()->item( insertAt-1 )->text() == s ) {
      //TODO: problem may appear if dictionary is different
      wordBox->setCurrentItem( wordBox->currentItem()-1 );
      return;
    }
  }
  
  list->insertItem( newItem, insertAt);

  // Remove some items if more then historyLen
  while(list->count() > historyLen)
  {
    //Delete item either from the beginning or end of the history
    if( wordBox->currentItem() < (int)list->count()/2 )
      list->removeItem(list->count() - 1);
    else {
      list->removeItem(0);
      wordBox->setCurrentItem( wordBox->currentItem()-1 );
    }
  }

  updateNavigationButtons();
}


void ZBEDic::addToHistoryDelayed( const QString& word )
{  
  // If exact match, activate 3s timer, which adds the word to the history
  // fullyInitialized is needed because the timer should not be started from the
  // constructor
  if( fullyInitialized ) {
    wordToAddToHistory = word;
    addToHistoryTimer->start( 3000, true );
  }
  
}

// ============= Back / Forward navigation  ===========
void ZBEDic::updateNavigationButtons()
{
  int historyItem = wordBox->currentItem();
  bool isValid = historyItem < (int)wordBox->count() && historyItem != -1;
    
  toolButton[TB_Back]->setEnabled( isValid &&
    (!isCurrentHistoryItem( word->text(), dict ) || historyItem != wordBox->count()-1) );
  toolButton[TB_Forward]->setEnabled( isValid && historyItem > 0 );
}

  

void ZBEDic::navigationNext()
{
  saveScrollPosition();
  if( wordBox->currentItem() > 0 && wordBox->currentItem() < (int)wordBox->count()) {
    saveScrollPosition();
    wordBox->setCurrentItem( wordBox->currentItem()-1 );
  }
}

void ZBEDic::navigationPrevious()
{
  if( !isCurrentHistoryItem( word->text(), dict ) &&
    wordBox->currentItem() < (int)wordBox->count() ) { // If between history entries
    saveScrollPosition();
    wordBox->setCurrentItem( wordBox->currentItem() );
  } else if( wordBox->currentItem() < (int)wordBox->count()-1 ) {
    saveScrollPosition();
    wordBox->setCurrentItem( wordBox->currentItem()+1 );
  }
  
}

void ZBEDic::loadDictionaries(const QStringList& activeDicts)
{
  for( uint i = 0; i < activeDicts.count(); i++ ) {
    loadSingleDictionary( activeDicts[i], false );
  }
}

Dictionary *ZBEDic::loadSingleDictionary(const QString &fileName, bool checkIntegrity)
{
  Dictionary* d = Dictionary::create( fileName, checkIntegrity );
  assert( d != NULL );
    
  string s = d->getError();
  if (s != "") {
    int n = fileName.findRev('/');
    QString file = fileName.mid(n+1);
    QString path = fileName.left(n);
    QString ss = "File: " + file + "\nPath: " + path + 
      "\nError: " + s.c_str() +
      "\nThe file won't be loaded"
      "\nat next startup.";
    QMessageBox::warning(0, "zbedic: Error", ss);
    delete d;
    
    return NULL;
  } else {
    dictionaries.inSort(d);
    return d;
  }
}


QStringList ZBEDic::getFontNames() {
	FontDatabase fd;
	return fd.families();
}

void ZBEDic::updateFont(QFont &font) {
  fontHeight = QFontMetrics( font ).height();
  setFont( font );
}

void ZBEDic::changeFont(QString& fname, int fsize) {
	fontName = fname;
	fontSize = fsize;

	QSize dsize = descr->size();
	QSize lsize = wordList->size();

	descr->setFixedSize(dsize);
	wordList->setFixedSize(lsize);
	
	font = QFont(fontName, fontSize);
	updateFont(font);
	descr->setFont(font);
	update();

	descr->setText(descrText);
}

void ZBEDic::msgFind(QString word) {
	if (!word.isEmpty()) {
		findWord(word);
	}
}

void ZBEDic::msgFind(QString dictionary, QString word) {
	if (dictionary.isEmpty()) {
		return;
	}

	Dictionary* d = findDictionary(dictionary);

	if (d == 0) {
		return;
	}

	setDictionary(d);
	msgFind(word);
}

void ZBEDic::msgHandler(const QCString& msg, const QByteArray& data) {
	QDataStream stream(data, IO_ReadOnly);

//	printf("msgHandler: %s\n", (const char*) msg);

        bool messageRecognized = false;

	if (msg == "find(QString)") {
		QString word;

		stream >> word;
		msgFind(word);
                messageRecognized = true;
                
	} else if (msg == "find(QString,QString)") {
		QString dictionary;
		QString word;

		stream >> dictionary;
		stream >> word;
		msgFind(dictionary, word);
                messageRecognized = true;
        }

        if( messageRecognized ) {
          cancelButton->show();
          settingsButton->hide();
          show();
          raise();
        }
}

void ZBEDic::c1Handler(const QCString& msg, const QByteArray& data) {
//	printf("c1Handler: %s\n", (const char*) msg);
}

void ZBEDic::c2Handler(const QCString& msg, const QByteArray& data) {
//	printf("c2Handler: %s\n", (const char*) msg);
}


DictBrowser::DictBrowser(QWidget* parent, const char* name) : 
  QTextBrowser(parent, name)
{
  originalFrameStyle = frameStyle();
}

void DictBrowser::setSource(const QString& name) {
	emit anchor(name);
}

void DictBrowser::focusInEvent(QFocusEvent *e)
{
  QTextBrowser::focusInEvent( e );
  setFrameStyle( QFrame::Panel + QFrame::Plain );
  update();
}

void DictBrowser::focusOutEvent(QFocusEvent *e)
{
  QTextBrowser::focusOutEvent( e );
  setFrameStyle( originalFrameStyle );  
  update();
}

DescrFormatter::DescrFormatter() {
	showPron = false;
}

QString DescrFormatter::format(const QString& text, const QString& headword) {
	QString result;
	QString etag;

	headWord = headword;
	partOfSpeech = QString();
	pron = QString();
	ndef = 0;

	if (processBody(text, 0, etag, result) >= 0) {
		return result;
	} else {
		return text;
	}
}

int DescrFormatter::processBody(const QString& text, int idx, QString& etag, 
	QString& ret) {

	int n, i = idx;
	QString savePartOfSpeech;

	while ((n = text.find('{', i)) >= 0) {
		// append the body text to the result
		ret += text.mid(i, n - i);

		// find the end of the tag
		int k = text.find('}', n+1);
		if (k < 0) {
			return -1;
		}

		// get the tag and whether it is empty
		QString tag = text.mid(n+1, k-n-1);
		bool empty = false;

		if (tag[tag.length()-1] == '/') {
			empty = true;
			tag = tag.left(tag.length() - 1);
		}

		// if the tag is closing tag, check if it is
		// the one we are looking for
		if (tag[0] == '/') {
			tag = tag.right(tag.length() - 1);
			if (tag != etag) {
				// non-matching end tag -- error
				return -1;
			} else {
				// found it, return
				return k+1;
			}
		}
		i = k+1;

		if (tag == "s") {
			savePartOfSpeech = partOfSpeech;
			ndef=0;
		}

		QString val;

		if (!empty) {
			i = processBody(text, i, tag, val);
		}

		if (i < 0) {
			// ignore the tag
			i = k+1;
			continue;
		}

		if (tag == "hw") {
			ret += "<b>" + headWord + "</b>";
		} else if (tag == "ps") {
			partOfSpeech = val;
		} else if (tag == "s") {
			ret += "<b>" + headWord + "</b> ";
			if (!pron.isEmpty() && showPron) {
				ret += " /" + pron + "/ ";
			}

			if (!partOfSpeech.isEmpty()) {
				ret += "<i>" + partOfSpeech + "</i>";
			}

			if (ndef > 1) {
				ret += "<ol>" + val + "</ol><hr/>";
			} else {
				while ((k = val.find("<li>")) >= 0) {
					val.remove(k, 4);
				}

				while ((k = val.find("</li>")) >= 0) {
					val.remove(k, 5);
				}

				ret += val + "<hr/>";
			}
			partOfSpeech = savePartOfSpeech;
		} else if (tag == "ss") {
			int ti, tn;
			ret += "<li>";
			for(ti = 0; (tn = val.find('\n', ti)) >= 0; ti = tn+1) {
				ret += "<p>" + val.mid(ti, tn-ti) + "</p>";
			}

			ret += "<p>" + val.right(val.length() - ti) + "</p></li>";
			ndef++;
		} else if (tag == "ex") {
			ret += "<i>" + val + "</i>";
		} else if (tag == "sa") {
			ret += "<a href=\"" + val + "\">" + val + "</a>";
		} else if (tag == "pr") {
			pron = val;
		} else if (tag = "br") {
			ret += "<br/>";
		} else {
			return -1;
		}
	}

	ret += text.right(text.length() - i);

	return text.length();
}

void ZBEDic::keyPressEvent ( QKeyEvent * e )
{
//  printf( "key: %x, state: %x \n", e->key(), e->state() );
  if( e->key() == Key_Down && e->state() == ShiftButton )
    descr->scrollBy( 0, fontHeight );
  else if( e->key() == Key_Up && e->state() == ShiftButton )
    descr->scrollBy( 0, -fontHeight );
  else if( e->key() == Key_Home && e->state() == ControlButton )
    navigationPrevious();    
  else if( e->key() == Key_End && e->state() == ControlButton )
    navigationNext();    
  else if( e->key() == Key_Down && e->state() != ShiftButton) { // Select a next item from the word list
    int item = wordList->currentItem();
    item++;
    if( item < (int)wordList->count() ) {
      wordList->setCurrentItem( item );
    }
  } else if( e->key() == Key_Up && e->state() != ShiftButton) { // Select a previous item from the word list
    int item = wordList->currentItem();
    if( item != 0 ) {
      item--;
      if( item < 0 ) item = 0;
      wordList->setCurrentItem( item );
    }
  } else
    QMainWindow::keyPressEvent( e );
}


void ZBEDic::updateLayout( int wordListSize, int layoutMode )
{
  int stretchFactor;
  if( wordListSize == 0 ) stretchFactor = 4;
  else if( wordListSize == 1 ) stretchFactor = 2;
  else stretchFactor = 1;
  
  mainLayout->setStretchFactor( descr, stretchFactor );

  QBoxLayout::Direction dir;
  if( layoutMode == 1 ) dir = QBoxLayout::TopToBottom;
  else if( layoutMode == 2 ) dir = QBoxLayout::LeftToRight;
  else {
    dir = QApplication::desktop()->width() > QApplication::desktop()->height() ?
      QBoxLayout::LeftToRight : QBoxLayout::TopToBottom;
  }

  mainLayout->setDirection( dir );
  
}

void ZBEDic::fillInDictPopup()
{
  dictPopup->clear();
  for(unsigned int i = 0; i < dictionaries.count(); i++) {
    Dictionary* d = dictionaries.at(i);
    dictPopup->insertItem(QString(d->getName().c_str()), i, i);
    if( d == dict )
      dictPopup->setItemChecked(i, true);
  }
}

void ZBEDic::switchToPreviousDict()
{
  if( previousDict != QString::null ) {
    Dictionary *d = findDictionary( previousDict );
    if( d != NULL )
      setDictionary( d );
  }
}

          
void ZBEDic::setDocument(const QString& fileref)
{
    if ( fileref.isNull() )
	return;

    //If the dictionary is available, change to that dict
    for(unsigned int i = 0; i < dictionaries.count(); i++) {
      if( fileref == dictionaries.at(i)->getFileName().c_str() ) {
        setDictionary( dictionaries.at(i) );
        return;
      }
    }

    if( QMessageBox::information( this, "zbedic",
          "Should the selected dictionary be\n"
          "added to the list of active\n"
          "dictionaries? ",
          "Add", "Cancel", QString::null,
          0,      
          1 ) == 0 ) {

      Dictionary *d = loadSingleDictionary( fileref, true );
      if( d != NULL )
        setDictionary( d );
    }    
}

void WordBox::keyPressEvent ( QKeyEvent * e )
{
  // Ignore up / down keys to handle them in main window
//  printf( "Key press event\n" );
  if( e->key() == Key_Down || e->key() == Key_Up ||
    (e->state() == ShiftButton && (e->key() == Key_Home || e->key() == Key_End)) ) e->ignore();
  else if( e->key() == Key_Tab && e->state() == NoButton ) lineEdit()->selectAll();
  else QComboBox::keyPressEvent ( e );
}  

void ZBEDic::lookupSelection()
{
  if( !descr->hasSelectedText() ) {
    Global::statusMessage ( tr( "Select the text first!" ) );
    return;
  }

  addToHistoryTriggered();      // Allow for back navigation  
  saveScrollPosition();
  
  QString selection = descr->selectedText().stripWhiteSpace();
  if( updateWordLookup( selection, true, true ) )
    addToHistory( selection );
}
