/****************************************************************************
* zbedic.h
*
* Copyright (C) 2002 Latchesar Ionkov <lionkov@yahoo.com>
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
*
* This program is based on the kbedic dictionary by 
* Radostin Radnev <radnev@yahoo.com>.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#ifndef ZBEDIC_H
#define ZBEDIC_H

//#include <qmainwindow.h>
#include <qwidget.h>
#include <qtextbrowser.h>
#include <qlist.h>
#include <qlistbox.h>
#include <qcombobox.h>

#include "dictionary.h"

class QFont;
class QLabel;
class QLineEdit;
class QListBox;
class QClipboard;
class QComboBox;
class QAction;
class QToolButton;
class QTextBrowser;
class DescrFormatter;
class QBoxLayout;
class QTimer;

class HistoryItem;
class DictBrowser;

/**
 * A list of Dictionaries, which can be sorted.
 */
class DictList : public QList<Dictionary>
{
protected:
  int compareItems( QCollection::Item item1, QCollection::Item item2 )
    {
      Dictionary *dic1 = (Dictionary*)item1;
      Dictionary *dic2 = (Dictionary*)item2;
      return QString::compare( dic1->getName().c_str(), dic2->getName().c_str() );
    }
};


//class ZBEDic:public QMainWindow {
class ZBEDic:public QWidget {
Q_OBJECT

public:
        ZBEDic(QWidget *parent = 0, const char *name = 0, WFlags f = WType_TopLevel );
	virtual ~ZBEDic();

protected:
	int processDescription(QString&, int, QString&, QString&, QString&);
	void addToHistory(const QString&);
        void addToHistoryDelayed( const QString& word );
	void findWord(const QString& text);
	Dictionary* findDictionary(const QString& dict);
	void setDictionary(Dictionary* d);
        void loadDictionaries(const QStringList& activeDicts);
        Dictionary *loadSingleDictionary(const QString &fileName, bool checkIntegrity);
	QStringList getFontNames();
        void updateFont(QFont &font);
	void changeFont(QString& fname, int fsize);
	void msgFind(QString word);
	void msgFind(QString dict, QString word);
        void updateLayout( int wordListSize, int layoutMode );
        void fillInDictPopup();
        bool isCurrentHistoryItem( const QString &word, const Dictionary *dic = NULL );
        HistoryItem *getCurrentHistoryItem();
        void saveScrollPosition();
        void gotoHistoryItem( HistoryItem *item );
        bool updateWordLookup( const QString &newWord, bool updateWordList = TRUE, bool updateWordBox = TRUE );
        void updateNavigationButtons();

        void keyPressEvent( QKeyEvent * e );
        void closeEvent( QCloseEvent * e );
        
public slots:
        void setDocument(const QString& fileref);
//        virtual void polish();

protected slots:
	void wordChanged(const QString& text);
	void listPick(const QString& text);
	void copy();
	void paste();
//	void settings();
	void cancel();
	void showDictionaries();
	void seeAlso(const QString& text);
	void selectDictionary(int idx);
	void msgHandler(const QCString& msg, const QByteArray& data);
	void c1Handler(const QCString& msg, const QByteArray& data);
	void c2Handler(const QCString& msg, const QByteArray& data);
        void addToHistoryTriggered();
//        void historyItemActivated(int idx);          
        void switchToPreviousDict();
	void navigationNext();
	void navigationPrevious();
        void clearHistory();
        void lookupSelection();
  
protected:
	DictList dictionaries; // list of available dictionaries
	Dictionary* dict; // current dictionary
	DescrFormatter* descrFormatter; // description formatter
        bool onWordLookup;    // set if in the process of updating word lookup 
	QString descrWord;
	QString descrText;

        QString previousDict; //Used for toggling between two recent dictionaries

	QFont font;
	QLineEdit *word;
	QComboBox* wordBox;
	QListBox *wordList;
	DictBrowser *descr;
	QClipboard *systemClipboard;
	QPopupMenu *dictPopup;
	QToolButton* dictButton;
//	QToolButton* settingsButton;
	QToolButton* cancelButton;
        QBoxLayout* mainLayout;

        enum ToolButtons {
            TB_SwitchDictionaries = 0,
            TB_Back,
            TB_Forward,
            TB_LookupSelection,
            TB_Copy,
            TB_Paste,
            TB_count
          };
        
        QToolButton *toolButton[TB_count];
        
        QTimer *addToHistoryTimer;
        QString wordToAddToHistory;
        
	unsigned int listLen; // number of words to show in the word list
	unsigned int historyLen; // number of words to remember in the history
	QString fontName; // font name
	int fontSize; // font size
        int fontHeight;        // Height of the font in pixels, used for scrolling
	QStringList dictPaths; // list of directories that contain dictionaries
	bool showPron; // show pronounciation
        int wordListSize; // Relative size of the wordList widget
        int layoutMode; // Horizontal, Vertical or automatic layout of wordlist and description
        bool fullyInitialized; //Set after leaving the constructor - prevents timer from starting in constructor
        
};


class WordList: public QListBox 
{
public:
  WordList( QWidget *parent ) :QListBox( parent )
    {
    }

  /* Solves layout problems */
  virtual QSize sizeHint () const
    {
      return QSize();
    }  
};


class DictBrowser : public QTextBrowser {
Q_OBJECT
  int originalFrameStyle;

public:
	DictBrowser(QWidget* parent=0, const char* name=0);
	virtual void setSource(const QString& name);

  /* Solves layout problems */
        virtual QSize sizeHint () const
          {
            return QSize();
          }
        
protected:
	virtual void focusInEvent(QFocusEvent*);
	virtual void focusOutEvent(QFocusEvent*);

signals:
	void anchor(const QString& name);
};

class DescrFormatter {
public:
	DescrFormatter();

	QString format(const QString& text, const QString& headword);
	bool getShowPron() {
		return showPron;
	}

	void setShowPron(bool v) {
		showPron = v;
	}

protected:
	QString partOfSpeech;
	QString headWord;
	QString pron;
	int ndef;
	bool showPron;

	int processBody(const QString& text, int idx, QString& endTag,
		QString& ret);
};


/**
 * An item in a history combo box. The item holds both a word and a
 * dictionary for that word.
 */
class HistoryItem: public QListBoxText
{
  QString  dict;
  int scrollPosition;           /* Allows to scroll to the same position as before clicking on a link */
  
public:
  HistoryItem( const QString &text, const QString &dict ):
    QListBoxText( text ), dict( dict ), scrollPosition( 0 )
    {      
    }
  
  const QString &getDictionary() const
    {
      return dict;
    }

  void setScrollPosition( int newPos )
    {
      scrollPosition = newPos;
    }

  int getScrollPosition()
    {
      return scrollPosition;
    }
};


/**
 * A combo box for entering words to translate.
 */
class WordBox: public QComboBox
{
Q_OBJECT

public:
  WordBox( QWidget *parent ) : QComboBox( parent )
  {
  }

protected:

  void keyPressEvent ( QKeyEvent * e );

  // This prevents from hiding tool buttons on 5x00 when bigger font is selected
  QSize sizeHint () const
    {
      QSize cbSize = QComboBox::sizeHint();
      cbSize.setWidth( 70 );
      return cbSize;
    }  
  
public slots:
  void activatePopup()
  {
    popup();
  }

  
};


#endif
